from . import special_values as special_values
from _typeshed import Incomplete

type_regexes: Incomplete

def assign_value(final_struct, key, value, valueType): ...
def parse_iso8601_date(input_string): ...
def parse_iso8601_datetime(input_string): ...
def parse_iso8601_instant(input_string): ...
def parse_iso8601_time(input_string): ...
def parse_flexible_address(address): ...
def caret_delimited_string_to_codeableconcept(caret_delimited_str): ...
def caret_delimited_string_to_coding(caret_delimited_str): ...
def string_to_quantity(quantity_str): ...
def parse_human_name(value): ...
