from .model.cohort_data_entity import CohortData as CohortData
from .model.resource_definition_entity import ResourceDefinition as ResourceDefinition
from .model.resource_link_entity import ResourceLink as ResourceLink

def read_xlsx_and_process(file_path): ...
def process_sheet_resource_definitions(sheet) -> list[ResourceDefinition]: ...
def process_sheet_resource_links(sheet) -> list[ResourceLink]: ...
def process_sheet_patient_data_revised(sheet, resource_definition_entities): ...
